@@ -19,6 +19,123 @@ except ImportError:
     # Python 3.
     import urllib.parse as urlparse
 
+ovstools_script = '''#!/bin/bash
+#Transfer the ip address from $bridge to $physicial interface.
+# INPUT:
+#     br: bridge name
+#     ifname: interface name
+# OUTPUT:
+#     null
+function ovsif_takeover()
+{
+    local br=$1
+    local ifname=$2
+    local ip4=""
+    local ip6=""
+    local mac=""
+    local route=""
+
+    ip4=$(ip -o -4 addr list $ifname | awk '{print $4}')
+    # TODO:IPV6
+    ip6=$(ip -o -6 addr list $ifname | awk '{print $4}')
+    mac=$(ip link show $ifname | grep "link/ether" | awk '{print $2}')
+    IFS=$'\n'
+    route=($(ip route show | grep $ifname | grep -v link | sed "s/$ifname/$br/g"))
+
+    echo "Take over $ip4 from $ifname to $br."
+    ip addr flush dev $ifname
+    ovs-vsctl add-port $br $ifname
+    ovs-vsctl set bridge $br other_config:hwaddr=$mac
+    ip link set $br up
+    ip address add $ip4 dev $br
+
+    # transfer the route from $ifname to $br
+    for line in ${route[@]};
+    do
+        #echo "ip route add $line"
+        eval ip route add $line
+    done
+
+    return 0
+}
+
+#Restore the ip address from $physicial interface to $bridge.
+# INPUT:
+#     br: bridge name
+#     ifname: interface name
+# OUTPUT:
+#     null
+function ovsif_restore()
+{
+    local br=$1
+    local ifname=$2
+    local ip4=""
+    local ip6=""
+    local route=""
+
+    ip4=$(ip -o -4 addr list $br | awk '{print $4}')
+    # TODO:IPV6
+    ip6=$(ip -o -6 addr list $ifname | awk '{print $4}')
+    IFS=$'\n'
+    route=($(ip route show | grep $br | grep -v link | sed "s/$br/$ifname/g"))
+
+    echo "restore $ip4 from $br to $ifname."
+    ip addr flush dev $br
+    ovs-vsctl del-port $br $ifname
+    ip address add $ip4 dev $ifname
+
+    # transfer the route from $ifname to $br
+    for line in ${route[@]};
+    do
+        #echo "ip route add $line"
+        eval ip route add $line
+    done
+
+    return 0
+}
+
+function ovsif_help()
+{
+    echo "Usage:"
+    echo "    sh $0 takeover br ifname."
+    echo "    sh $0 restore br ifname."
+}
+
+# The main function for ovsif takeover
+# INPUT: $@: All the parameters
+# OUTPUT: null
+# RETURN: 0 if succeed, or else
+function main()
+{
+    local params=""
+
+    if test $# -eq 0
+    then
+        ovsif_help
+        return 0
+    fi
+
+    case "$1" in
+        takeover)
+            shift
+            ovsif_takeover "$@" ;;
+        restore)
+            shift
+            ovsif_restore "$@" ;;
+        *)
+            shift
+            ovsif_help ;;
+    esac
+}
+
+# Variables must be declared explicitly
+set -u
+main "$@"
+exit $?
+
+# vim: ts=4 sw=4 cindent expandtab
+'''
+
 
 class OvsHost(linux.LinuxDevice):
     '''
@@ -107,7 +224,8 @@ class OvsHost(linux.LinuxDevice):
         self.cmd(command) #execut the command.
 
         # delete the current offlow when it's created
-        if self["fail-mode"] == "secure":
+        #if self["fail-mode"] == "secure":
+        if self.cmd("ovs-vsctl get-fail-mode %s" %(name)).strip() == "secure":
             self.ofctl_add_flow(name, "priority=0,actions=drop")
 
         if ip:
@@ -127,7 +245,7 @@ class OvsHost(linux.LinuxDevice):
         Add self-port and do some configuration if necessary.
         '''
         name = kwargs.get("name", "tmpname")
-        command = 'ovs-vsctl add-port %s %s' %(bridge_name, name)
+        command = 'ovs-vsctl --may-exist add-port %s %s' %(bridge_name, name)
 
         # parameter ip/mac is not valid, won't be included in the add-port
         port_options = {}
@@ -141,8 +259,9 @@ class OvsHost(linux.LinuxDevice):
         self.cmd(command) #execut the command.
 
         #If it's a normal interface, flush it to release its original ip address.
-        if (kwargs.get("type", "normal") == "normal"):
-            self.cmd("ip addr flush dev %s" %(name))
+        #if (kwargs.get("type", "normal") == "normal"):
+        #    #self.cmd("ip addr flush dev %s" %(name))
+        #    self.cmd("sh ovstools.sh takeover %s %s" %(bridge_name, name))
 
         macaddr = self.ovs_get_interface(name, "mac_in_use").strip('"')
         kwargs["mac"] = macaddr
@@ -152,7 +271,8 @@ class OvsHost(linux.LinuxDevice):
             self.cmd('ip link set %s up' %(name))
             self.cmd('ip address add %s dev %s' %(kwargs["ip"], name))
             #self.cmd('ovs-appctl ovs/route/add %s %s' %(kwargs["ip"], name))
-            if self["fail-mode"] == "secure":
+            #if self["fail-mode"] == "secure":
+            if self.cmd("ovs-vsctl get-fail-mode %s" %(bridge_name)).strip() == "secure":
                 # Confiure the default offlow for self port with IP.
                 ipaddr = ipaddress.ip_interface(kwargs["ip"]).ip
                 self.ofctl_add_flow(bridge_name,
@@ -160,6 +280,30 @@ class OvsHost(linux.LinuxDevice):
                     'priority=10000,icmp,nw_dst=%s,icmp_type=8,icmp_code=0,actions=push:"NXM_OF_ETH_SRC[]",push:"NXM_OF_ETH_DST[]",pop:"NXM_OF_ETH_SRC[]",pop:"NXM_OF_ETH_DST[]",push:"NXM_OF_IP_SRC[]",push:"NXM_OF_IP_DST[]",pop:"NXM_OF_IP_SRC[]",pop:"NXM_OF_IP_DST[]",load:"0xff->NXM_NX_IP_TTL[]",load:"0->NXM_OF_ICMP_TYPE[]",in_port' %(ipaddr),
                     )
 
+    def takeover_interface (self, bridge_name, ifname):
+        '''
+        takeover the @ifname to @bridge_name
+        '''
+        filename = '.netdevice_ovstools.sh'
+        if self.cmd("[ -f ~/%s ] || echo no" %(filename)).strip() == "no":
+            with open(filename, 'w') as f:
+                f.write(ovstools_script)
+            self.put_file(filename, "~/")
+
+        self.cmd("sh ~/%s takeover %s %s" %(filename, bridge_name, ifname))
+
+    def no_takeover_interface (self, bridge_name, ifname):
+        '''
+        takeover the @ifname to @bridge_name
+        '''
+        filename = '.netdevice_ovstools.sh'
+        if self.cmd("[ -f ~/%s ] || echo no" %(filename)).strip() == "no":
+            with open(filename, 'w') as f:
+                f.write(ovstools_script)
+            self.put_file(filename, "~/")
+
+        self.cmd("sh ~/%s restore %s %s" %(filename, bridge_name, ifname))
+
     def add_device (self, bridge_name, *args, **kwargs):
         '''
         connect devices to ovs, now only support container.
@@ -182,7 +326,8 @@ class OvsHost(linux.LinuxDevice):
                 self.devices.append(arg)
 
                 # Configure offlow for the same subnet
-                if self["fail-mode"] == "secure":
+                #if self["fail-mode"] == "secure":
+                if self.cmd("ovs-vsctl get-fail-mode %s" %(bridge_name)).strip() == "secure":
                     ipaddr = ipaddress.ip_interface(arg["ip"])
                     self.ofctl_add_flow(bridge_name,
                         "priority=10000,arp,arp_tpa=%s,actions=output:%s"
@@ -191,8 +336,6 @@ class OvsHost(linux.LinuxDevice):
                         "priority=10001,dl_dst=%s,actions=output:%s" 
                         %(arg["mac"], arg["ofport"]),
                         )
-
-
             elif dtype == "vm":
                 self.__add_vm(bridge_name, arg)
             else:
@@ -292,14 +435,29 @@ class OvsHost(linux.LinuxDevice):
         # 3)add the tap port, and add the flow
         for p in vm.get("port", []):
             device = self._util_dictionary_parse(p["-device"].split(","))
+            netdev = self._util_dictionary_parse(p["-netdev"].split(","))
             if "vhost-user" in p["-netdev"]:
+                br = bridge_name
                 pass
             elif "tap" in p["-netdev"]:
                 # type: tap
-                netdev = self._util_dictionary_parse(p["-netdev"].split(","))
-                port_name = netdev.get("ifname", "tmp")
+                #netdev = self._util_dictionary_parse(p["-netdev"].split(","))
+                port_name = netdev.get("ifname", netdev.get("id", "id0"))
                 ovs_port = {"name": port_name}
-                self.add_port(bridge_name, ovs_port)
+                br = netdev.get("br", "br0") #default=br0
+                if (netdev.get("script", "/etc/qemu-ifup") == "no"):
+                    # There is no user-defined script, we add it by ourselves:
+                    self.cmd("ip link set %s up" %(port_name))
+                    self.add_port(br, ovs_port)
+            elif "bridge" in p["-netdev"]:
+                # type: tap
+                #netdev = self._util_dictionary_parse(p["-netdev"].split(","))
+                port_name = "%s_%s" %(vm["-name"], netdev.get("id", "tmp"))
+                ovs_port = {"name": port_name}
+                br = netdev.get("br", "br0") #default=br0
+                # qemu would add the port automatically for tap interface.
+                self.cmd("ip link set %s up" %(port_name))
+                self.add_port(br, ovs_port) #default=br0
             else:
                 # Now only vhost-user and tap will be added into OVS
                 continue
@@ -310,9 +468,10 @@ class OvsHost(linux.LinuxDevice):
             self.devices.append(p)
 
             # Configure offlow for the same subnet
-            if self["fail-mode"] == "secure":
+            #if self["fail-mode"] == "secure":
+            if self.cmd("ovs-vsctl get-fail-mode %s" %(br)).strip() == "secure":
                 ipaddr = ipaddress.ip_interface(p["ip"])
-                self.ofctl_add_flow(bridge_name,
+                self.ofctl_add_flow(br,
                     "priority=10000,arp,arp_tpa=%s,actions=output:%s"
                     %(ipaddr.ip, p["ofport"]),
                     # We always know the mac/port mapping, so add it.
@@ -325,7 +484,8 @@ class OvsHost(linux.LinuxDevice):
         Add some flows remote devices(*args): 从本主机上到达所有*args的包，全
         部经由vtep
         '''
-        if self["fail-mode"] != "secure":
+        #if self["fail-mode"] != "secure":
+        if self.cmd("ovs-vsctl get-fail-mode %s" %(bridge_name)).strip() != "secure":
             #Don't need configure vtep explicitly in standalone mode.
             return
         for arg in args:
@@ -341,6 +501,11 @@ class OvsHost(linux.LinuxDevice):
         '''
         Add some flows to ofproto
         '''
+        #if self["fail-mode"] != "secure":
+        if self.cmd("ovs-vsctl get-fail-mode %s" %(bridge_name)).strip() != "secure":
+            #Don't need configure gateway explicitly in standalone mode.
+            return
+
         #self.log("gw mac: %s" %(gw["mac"]))
         #self.log("mac: %s" %(self.macs))
         if (gw["mac"] not in self.macs):
@@ -354,6 +519,7 @@ class OvsHost(linux.LinuxDevice):
                     'priority=1000,icmp,nw_dst=%s,icmp_type=8,icmp_code=0,actions=push:"NXM_OF_ETH_SRC[]",push:"NXM_OF_ETH_DST[]",pop:"NXM_OF_ETH_SRC[]",pop:"NXM_OF_ETH_DST[]",push:"NXM_OF_IP_SRC[]",push:"NXM_OF_IP_DST[]",pop:"NXM_OF_IP_SRC[]",pop:"NXM_OF_IP_DST[]",load:"0xff->NXM_NX_IP_TTL[]",load:"0->NXM_OF_ICMP_TYPE[]",in_port' %(ipaddr))
 
         for d in args:
+            #self.log("d: %s" %(d))
             if (d["mac"] in self.macs):
                 # ONly change the mac and ttl on the same host.
                 self.ofctl_add_flow(bridge_name,
@@ -365,7 +531,8 @@ class OvsHost(linux.LinuxDevice):
         '''
         It's the same as add_vtep(), only different in the name.
         '''
-        if self["fail-mode"] != "secure":
+        #if self["fail-mode"] != "secure":
+        if self.cmd("ovs-vsctl get-fail-mode %s" %(bridge_name)).strip() != "secure":
             #Don't need configure out_if explicitly in standalone mode.
             return
         for arg in args:
